#!/usr/bin/env python
# -*- encoding: latin1 -*-
#
# Copyright 2007 Scott Kirkwood

from _plugin import ClipboardPlugin, TestPlugin
import re

def create_plugin():
  return TabsToBars()

class TabsToBars(ClipboardPlugin):
  def __init__(self):
    ClipboardPlugin.__init__(self)
    self.max_col_width = 70
    
  def name(self):
    return 'Wikify Table'

  def description(self):
    return 'Wikify a table separated by tabs'

  def convert(self, text):
    """ Convert tab delimited data, from say, OpenOffice.calc into a wiki format
    To make it more editable in the wiki afterwards tries to make all the columns
    fit the text below it.
    
    Returns: Text
    """
    
    if len(text) == 0:
      return self._ret_result('Nothing to do', False)
    
    lines = text.split('\n')
    for rowIndex, line in enumerate(lines):
      lines[rowIndex] = [x.rstrip() for x in line.split('\t')]
    
    if len(lines) <= 1:
      return self._ret_result("Not likely a table", False)
    
    max_width = []
    for line in lines:
      for colIndex, col in enumerate(line):
        if colIndex >= len(max_width):
          max_width.append(0)
        curColLen = len(col)
        if curColLen > max_width[colIndex]:
          max_width[colIndex] = curColLen
        if max_width[colIndex] > self.max_col_width:
          max_width[colIndex] = self.max_col_width
    
    ret = []
    last_line = lines[-1]
    last_line_is_empty = True
    for col in last_line:
      if len(col) > 0:
        last_line_is_empty = False
        break
    if last_line_is_empty:
      lines = lines[:-1]
    
    for rowIndex, line in enumerate(lines):
      newLine = []
      for colIndex, col in enumerate(line):
        line = '%-*s|' % (max_width[colIndex], col)
        if colIndex == 0:
          line = '|' + line
        newLine.append(line)
      
      ret.append(''.join(newLine))
      message = "Converted table with %d columns and %d lines" % (len(max_width), 
        len(ret))
    
    return self._ret_result(message, True, '\n'.join(ret))


class TestTabsToBars(TestPlugin):
  def setUp(self):
    self.instance = TabsToBars()
    
  def test_good(self):
    good_samples = [
      ("""One\tTwo\tThree
Col1\tCol2\tCol3""",
       """|One |Two |Three|
|Col1|Col2|Col3 |"""),
      ("""One\tTwo\tThree
Col1\tCol2\tCol3
""",
       """|One |Two |Three|
|Col1|Col2|Col3 |"""),
      ("""One\tTwo\tThree
Col1\tCol2\tCol3
      """,
       """|One |Two |Three|
|Col1|Col2|Col3 |"""),
      ("""1\t1\t1
1\t1\t1
      """,
       """|1|1|1|
|1|1|1|"""),
    ]
    self.verify_good_samples(good_samples, "Converted table")

  def test_bad(self):
    bad_samples = [
      "",
    ]
    self.verify_bad_samples(bad_samples)
    
if __name__ == "__main__":
  import unittest
  unittest.main()



#!/usr/bin/env python
# -*- encoding: latin1 -*-
#
# Copyright 2007 Scott Kirkwood
import wx
import subprocess
import optparse
import re

"""
Clipboard modifier is a wxPython app with a simple purpose.  
When brought to the front, it'll modify the clipboard using a routine that you supply.
An example of this is talking the string "Hello" and putting it in quotes '"Hello'", which
becomes more interesting when you have several lines and some quotes within.
The program is exensible, any .py files found in the plugins subdirectory will get
loaded automatically. For each plugin we need a module level function called 

Note to self:
Use the same naming conventions as wxPython in this file.
"""
class ClipboardModifierApp(wx.App):
  def init_vars(self):
    self.methods = []
    self.grabbedClipboard = ''
    self.sendClipboard = ''
    self.beforeTemplate = "From: %0d bytes"
    self.afterTemplate = "To: %0d bytes"
    self.unchanged_after = "Unchanged"
    self.class_state = {}
    
  def OnInit(self):
    self.init_vars()
    dialog = wx.Frame(None, -1, "Clipboard Modifier")
    dialog.Show(True)
    self.SetTopWindow(dialog)
    self.FindModules()
    
    self.SetupControls(dialog)
    return True

  def SetupControls(self, dialog):
    vsizer = wx.BoxSizer(wx.VERTICAL)
    
    hsizer = wx.BoxSizer(wx.HORIZONTAL)
    self.dialog = dialog 
    self.BeforeSize = wx.TextCtrl(dialog, -1, self.beforeTemplate % (0), size=(-1,20), style=wx.TE_READONLY)
    self.BeforeTooltip = wx.ToolTip("")
    self.BeforeTooltip.SetDelay(1)
    self.BeforeSize.SetToolTip(self.BeforeTooltip)
    hsizer.Add(self.BeforeSize, 1, wx.ALL, 5)
    
    self.AfterSize = wx.TextCtrl(dialog, -1, self.afterTemplate % (0), size=(-1, 20), style=wx.TE_READONLY)
    self.AfterTooltip = wx.ToolTip("")
    self.AfterTooltip.SetDelay(1)
    self.AfterSize.SetToolTip(self.AfterTooltip)
    hsizer.Add(self.AfterSize, 1, wx.ALL, 5)
    
    vsizer.Add(hsizer, 0, wx.ALL | wx.EXPAND, 5)
    
    # First one has this style, the remainder have a style of 0
    self.combo = wx.ComboBox(dialog, -1, style=wx.CB_DROPDOWN | wx.CB_READONLY)
    for class_instance in self.clipoard_methods:
      title = class_instance.name()
      self.combo.Append(title)

    self.Bind(wx.EVT_COMBOBOX, self.SetClassToUse, self.combo)
    vsizer.Add(self.combo, 0, wx.ALL | wx.EXPAND, 5)
    self.textDescription = wx.StaticText(dialog, -1, size=(-1, 40), style=wx.TE_READONLY)
    vsizer.Add(self.textDescription, 1, wx.ALL | wx.EXPAND, 10)
    #self.textDescription.Wrap(100)
    self.vsizer = vsizer

    self.plugin_controls_sizer = wx.BoxSizer(wx.VERTICAL)
    self.vsizer.Add(self.plugin_controls_sizer, 0, wx.ALL | wx.EXPAND, 10)

    self.sb = dialog.CreateStatusBar()
    dialog.SetSizer(vsizer)
    self.combo.SetFocus()
    self.SetDefaultClass(0)
    wx.EVT_ACTIVATE(self, self.OnActivate)

  def SetDefaultClass(self, index):
    self.class_to_use = self.clipoard_methods[0]
    self.combo.SetValue(self.class_to_use.name())
    self.SetClassInstance(self.clipoard_methods[0])
    
  def SetClassToUse(self, event):
    """User changed which class to use.

    Also forces a convert with the new class.
    """
    classTitle = event.GetString()
    for class_instance in self.clipoard_methods:
      if class_instance.name() == classTitle:
       self.SetClassInstance(class_instance)
       return

  def SetClassInstance(self, class_instance):
    self.class_state[self.class_to_use.filename] = self.class_to_use.get_state()
    self.class_to_use = class_instance
    self.textDescription.SetLabel(class_instance.description())
    self.ModifyText()
    # This bit of magic resizes the StaticText() so that it wraps if it needs to.
    sizer = self.textDescription.GetContainingSizer()
    sizer.RecalcSizes()
    self.plugin_controls_sizer.DeleteWindows()
    for ctrl in self.class_to_use.control_iter(self.dialog):
      if ctrl:
        self.plugin_controls_sizer.Add(ctrl, 0, wx.ALL | wx.EXPAND, 0)
    if self.class_to_use.filename in self.class_state:
      self.class_to_use.restore_state(self.class_state[self.class_to_use.filename])
    
  def GrabClipboard(self):
    """ Grab the clipboard data """
    if wx.TheClipboard.Open():
      data = wx.TextDataObject()
      success = wx.TheClipboard.GetData(data)
      self.grabbedClipboard = data.GetText()
      wx.TheClipboard.Close()
    
    return self.grabbedClipboard
  
  def UpdateClipboard(self, text):
    """ Overwrite the clipboard data """
    if wx.TheClipboard.Open():
      data = wx.TextDataObject(text)
      wx.TheClipboard.SetData(data)
      wx.TheClipboard.Close()
      return True
    return False
  
  def RunFunction(self, text, func):
    """ Run the function """
    results = func.convert(text)
    return func.message(), results

  def ModifyText(self):
    message = 'Ok'
    if len(self.grabbedClipboard) > 0:
      message, self.sendClipboard = self.RunFunction(self.grabbedClipboard, self.class_to_use)
    else:
      message = "No clipboard data"
    
    bytes_before = len(self.grabbedClipboard)
    bytes_after = len(self.sendClipboard)
    beforeMessage = self.beforeTemplate % (bytes_before)
    self.BeforeSize.SetValue(beforeMessage)
    afterMessage = ''
    if bytes_after == 0:
      afterMessage = self.unchanged_after
      self.AfterTooltip.SetTip(self.TrimTip(afterMessage))
    else:
      afterMessage = self.afterTemplate % (bytes_after)
      self.AfterTooltip.SetTip(self.TrimTip(self.sendClipboard))
    self.AfterSize.SetValue(afterMessage)
      
    self.BeforeTooltip.SetTip(self.TrimTip(self.grabbedClipboard))
    
    self.sb.SetStatusText(message,0)
    return message

  def TrimTip(self, text):
    """ Make sure the tip isn't too big """
    max = 2048
    if len(text) > max:
      return text[0:2048]+"..."
    return text
    
  def OnActivate(self, event):
    message = 'Ok'
    if event.GetActive():
      self.GrabClipboard()
      message = self.ModifyText()
    else:
      if len(self.sendClipboard) > 0:
        if not self.UpdateClipboard(self.sendClipboard):
          message = "Unable to update the clipboard"

    event.Skip()

  def FindModules(self):
    self.clipoard_methods = []

    pluginfiles = self._GetPluginFiles()
    self.import_plugin("plugins.donothing")
    for filename in pluginfiles:
      if filename.endswith('.py') \
          and not filename.startswith('_')  \
          and not filename == 'donothing.py':
        import_text = 'plugins.'  + filename.replace('.py', '')
        self.import_plugin(import_text)

  def import_plugin(self, import_text):
    import plugins
    
    print import_text
    try:
      __import__('clipboardmodifier.' + import_text)
    except:
      __import__(import_text) # This works when developing
    new_plugin = eval(import_text + '.create_plugin()')
    self.clipoard_methods.append(new_plugin)
    setattr(new_plugin, 'parent', self)
    setattr(new_plugin, 'filename', import_text)
  
  def _GetPluginFiles(self):
    import os
    import plugins
    
    plugindir = os.path.join(re.sub('__init__.py.?', '', plugins.__file__))
    ret = os.listdir(plugindir)
    return ret

class MyApp(wx.App):
    def OnInit(self):
        frame = ClipboardModifierApp(None, -1, 'Clipboard Modifier')
        frame.Show(True)
        self.SetTopWindow(frame)
        return True
        
def RunWxApp():
  app = ClipboardModifierApp(0)
  app.MainLoop()
  
if __name__ == '__main__':
  import optparse
  
  parser = optparse.OptionParser()
  (options, args) = parser.parse_args()

  RunWxApp()



#!/usr/bin/env python
# -*- encoding: latin1 -*-
#
# Copyright 2007 Scott Kirkwood

from _plugin import ClipboardPlugin, TestPlugin
import re
import subprocess
import wx

def create_plugin():
  return RunApp()

class RunApp(ClipboardPlugin):
  def __init__(self):
    ClipboardPlugin.__init__(self)
    self.application = "sort"
    self.arguments = "-n"
    
  def name(self):
    return 'Run a program'
    
  def description(self):
    return 'Run a program that accepts a stream from stdin and returns a stream in stdout'
    
  def convert(self, text):
    """ Convert the program by streaming to and from the application
      
    Returns: Text
    """
    args = self.application + ' ' + self.arguments
    p = subprocess.Popen(args, stdin=subprocess.PIPE,   
        stdout=subprocess.PIPE, close_fds=True, shell=True)
    input = p.stdin
    output = p.stdout
    input.write(text)
    input.close()
    ret = []
    for line in output:
      ret.append(line)
    return self._ret_result("Converted", True, ''.join(ret))

  def control_iter(self, dialog):
    yield wx.StaticText(dialog, -1, "Program")
    text_ctrl = wx.TextCtrl(dialog, -1, self.application)
    dialog.Bind(wx.EVT_TEXT, self.ApplicationChanged, text_ctrl)
    yield text_ctrl
    text_ctrl = wx.TextCtrl(dialog, -1, self.arguments)
    dialog.Bind(wx.EVT_TEXT, self.ArgumentsChanged, text_ctrl)
    yield text_ctrl

  def ApplicationChanged(self, event):
    self.application = event.GetString()

  def ArgumentsChanged(self, event):
    self.arguments = event.GetString()

class TestRunApp(TestPlugin):
  def setUp(self):
    self.instance = RunApp()
    self.instance.application = 'sort'
    self.instance.arguments = '-n'
    
  def test_good(self):
    good_samples = [
      ("11\n2\n3\n",
       '2\n3\n11\n'),
    ]
    self.verify_good_samples(good_samples, "Converted")
    
if __name__ == "__main__":
  import unittest
  unittest.main()



#!/usr/bin/env python
# -*- encoding: latin1 -*-
#
# Copyright 2007 Scott Kirkwood

from _plugin import ClipboardPlugin, TestPlugin
import re

def create_plugin():
  return ToJava()

class ToJava(ClipboardPlugin):
  def __init__(self):
    ClipboardPlugin.__init__(self)
    self.plus_at_start = True
    
  def name(self):
    return 'Convert line(s) to Java Strings'
    
  def description(self):
    return 'Convert lines(s) into java strings escaping double quotes as needed.'
    
  def convert(self, text):
    """ Convert Lines into Java strings
      
    Returns: Text
    """
    
    if len(text) == 0:
      return self._ret_result('Nothing to do', False)
    
    lines = text.split('\n')
    
    last_line = lines[-1]
    last_line_is_empty = True
    for col in last_line:
      if len(col) > 0:
        last_line_is_empty = False
        break
    
    if last_line_is_empty:
      lines = lines[:-1]
    
    ret = []
    for rowIndex, line in enumerate(lines):
      newLine = line.replace('\\', '\\\\')
      newLine = newLine.replace('"', '\\"')
      ret.append('"%s"' % (newLine))
      
    message = "Put quotes around text" 
    return self._ret_result(message, True, '\n+'.join(ret))

class TestToJava(TestPlugin):
  def setUp(self):
    self.instance = ToJava()
    
  def test_good(self):
    good_samples = [
      ("String",
       '"String"'),
      (r'\n',
       r'"\\n"'),
      ('"Double quote"',
       '"\\"Double quote\\""'),
      ('''Two
Lines''',
       '''"Two"
+"Lines"'''),
    ]
    self.verify_good_samples(good_samples, "Put quotes around text")

  def test_bad(self):
    bad_samples = [
      "",
    ]
    self.verify_bad_samples(bad_samples)
    
if __name__ == "__main__":
  import unittest
  unittest.main()



#!/usr/bin/env python
# -*- encoding: latin1 -*-
#
# Copyright 2007 Scott Kirkwood

from _plugin import ClipboardPlugin, TestPlugin
import re
import wx

def create_plugin():
  return ShowPythonVar()

class ShowPythonVar(ClipboardPlugin):
  def __init__(self):
    ClipboardPlugin.__init__(self)
    self.reset()

  def reset(self):
    self.template = "print '%(var)s = %%s' %% str(%(var)s)" 

  def name(self):
    return 'Show python var'
    
  def description(self):
    return 'Makes "%s" into "%s"' % ('my_var', self.template % dict(var='my_var'))
    
  def convert(self, text):
    """ Convert some text using the expression given
      
    Returns: Text
    """

    failed = False
    try:
      text = self.template % dict(var=text)
    except ValueError:
      failed = True
    if failed:
      return False
    else:
      return self._ret_result("Converted", True, text)

  def control_iter(self, dialog):
    yield wx.StaticText(dialog, -1, 'Template:')
    text_ctrl = wx.TextCtrl(dialog, -1, self.template)
    yield text_ctrl
    dialog.Bind(wx.EVT_TEXT, self.ChangedTemplate, text_ctrl)

  def ChangedTemplate(self, event):
    self.template = event.GetString()
    failed = False
    try:
      text = self.template % dict(var='bob')
    except ValueError:
      failed = True
    if not failed:
      self.parent.ModifyText()

class TestShowPythonVar(TestPlugin):
  def setUp(self):
    self.instance = ShowPythonVar()
    
  def test_good(self):
    good_samples = [
      ("my_var",
       "print 'my_var = %s' % str(my_var)"),
    ]
    self.verify_good_samples(good_samples, "Converted")
    
if __name__ == "__main__":
  import unittest
  unittest.main()



#!/usr/bin/env python
# -*- encoding: latin1 -*-
#
# Copyright 2007 Scott Kirkwood

from _plugin import ClipboardPlugin, TestPlugin
import re

def create_plugin():
  return LinesToCols()

class LinesToCols(ClipboardPlugin):
  def __init__(self):
    ClipboardPlugin.__init__(self)
    self.num_columns = 4

  def name(self):
    return 'Rows into columns/rows'

  def description(self):
    return 'Writely copies a table as a bunch of rows, make it into rows columns'

  def convert(self, text):
    """ Data that is in columns into something
    separated by quotes and commas
    
    Returns: Text
    """
    
    if len(text) == 0:
      return self._ret_result('Nothing to do', False)
    
    lines = text.split('\n')
    
    if len(lines) <= 1:
      return self._ret_result("Not likely a table", False)
    
    ret = []
    self.cur_row = []
    for line in lines:
      self._out_col(ret, line)
    self._append_row(ret)
    
    message = "Converted into %dx%d (row/cols)" % (len(ret), self.num_columns)
    return self._ret_result("Converted", True, '\n'.join(ret))
    
  def _out_col(self, ret, line):
    if len(self.cur_row) >= self.num_columns:
      self._append_row(ret)
    self.cur_row += line.lstrip().split('\t')
      
  def _append_row(self, ret):
      ret.append('\t'.join(self.cur_row))
      self.cur_row = []
    
class TestLinesToCols(TestPlugin):
  def setUp(self):
    self.instance = LinesToCols()
    
  def test_good(self):
    good_samples = [
      ("One\n\tTwo\n\tThree\nFour\nCol1\nCol2\nCol3\nCol4\n",
       "One\tTwo\tThree\tFour\nCol1\tCol2\tCol3\tCol4\n",),
      ("One\nTwo\nThree\nFour\nCol1\nCol2\nCol3\nCol4",
       "One\tTwo\tThree\tFour\nCol1\tCol2\tCol3\tCol4",),
      ("One\nTwo\nThree\nFour\nCol1\nCol2\nCol3",
       "One\tTwo\tThree\tFour\nCol1\tCol2\tCol3",),
    ]
    self.verify_good_samples(good_samples, "Converted")

  def test_bad(self):
    bad_samples = [
      "",
    ]
    self.verify_bad_samples(bad_samples)
    
if __name__ == "__main__":
  import unittest
  unittest.main()



#!/usr/bin/env python
# -*- encoding: latin1 -*-
#
# Copyright 2007 Scott Kirkwood

from _plugin import ClipboardPlugin, TestPlugin
import urllib
import wx

def create_plugin():
  return Text2Unicode()

class Text2Unicode(ClipboardPlugin):
  def __init__(self):
    ClipboardPlugin.__init__(self)
    self.from_unicode = False
    
  def name(self):
    return 'Text to escaped unicode.'
    
  def description(self):
    return 'Foreign characters translated to unicode style escapes.'
    
  def convert(self, text):
    """Convert text into unicode
      
    Returns: Text
    """
    new_text = text.encode('unicode-escape', 'backslashreplace')
    return self._ret_result("Converted", True, new_text)

  def control_iter(self, dialog):
    cb = wx.CheckBox(dialog, -1, label='From Unicode')
    cb.SetValue(self.from_unicode)
    dialog.Bind(wx.EVT_CHECKBOX, self.EvtCheckBox, cb)
    yield cb

  def EvtCheckBox(self, event):
    self.from_unicode = event.Checked()
    self.parent.ModifyText()

class TestUrl2Text(TestPlugin):
  def setUp(self):
    self.instance = Text2Unicode()
    
  def test_good(self):
    good_samples = [
      (u'Joćo',
      'Jo\\xe3o'),
    ]
    self.verify_good_samples(good_samples, "Converted")
    
if __name__ == "__main__":
  import unittest
  unittest.main()



#!/usr/bin/env python
# -*- encoding: latin1 -*-
#
# Copyright 2007 Scott Kirkwood

from _plugin import ClipboardPlugin, TestPlugin
import re
import wx

def create_plugin():
  return AmazonLink()
  
class AmazonLink(ClipboardPlugin):
  def __init__(self):
    ClipboardPlugin.__init__(self)
    self.reset()

  def reset(self):
    self.associate_id = 'softwareforusers'

  def name(self):
    return 'Amazon Associates URL'
  
  def description(self):
    return 'Convert an amazon URL so that it has the Associates ID in it'

  def convert(self, text):
    """ Convert an http:// link to an amazon style link
    
    Returns text (url)
    """
    base_url = 'http://www.amazon.com/exec/obidos/ASIN'
    re_asin = re.compile(r'/([A-Z0-9]{10})')
    args = re_asin.search(text)
    if not args:
      return self._ret_result("Didn't find 10 digit ASIN in link", False, "")
    asin = args.group(1)
    return self._ret_result("Created URL", True, '%s/%s?tag=%s' % (base_url, asin, self.associate_id))

  def control_iter(self, dialog):
    yield wx.StaticText(dialog, -1, "Amazon Associates ID:")
    text = wx.TextCtrl(dialog, -1, self.associate_id)
    dialog.Bind(wx.EVT_TEXT, self.EvtText, text)
    yield text

  def EvtText(self, event):
    self.associate_id = event.GetString()
    self.parent.ModifyText()

  def get_state(self):
    return "{'associate_id' : '%s'}" % self.associate_id

  def restore_state(self, text):
    adict = eval(text)
    self.associate_id = adict['associate_id']

class TestAmazonLink(TestPlugin):
  def setUp(self):
    self.instance = AmazonLink()
    
  def test_good(self):
    good_samples = [
      ("http://amazon.com/o/ASIN/0201485419/ref=s9_asin_image_2-hf_favarpcbss_2238_p/102-1722741-9600945?%5Fencoding=UTF8&coliid=I1B1K3406DXXD8&colid=3TDODZYOX7Y6Z&pf_rd_m=ATVPDKIKX0DER&pf_rd_s=center-2&pf_rd_r=07G7DSC9Q3E0C50S0FQM&pf_rd_t=101&pf_rd_p=279438701&pf_rd_i=507846",
      "http://www.amazon.com/exec/obidos/ASIN/0201485419?tag=softwareforusers"
      ),
      ("http://amazon.com/Nintendo-RVLSWCUSZ-Wii/dp/B0009VXBAQ/ref=pd_ys_qtk_shop/102-1722741-9600945?pf_rd_m=ATVPDKIKX0DER&pf_rd_s=center-1&pf_rd_r=1MXPWA8MWGFQ9K6G8JHT&pf_rd_t=1501&pf_rd_p=186414001&pf_rd_i=home",
      "http://www.amazon.com/exec/obidos/ASIN/B0009VXBAQ?tag=softwareforusers"
      ),
    ]
    self.verify_good_samples(good_samples, "Created URL")

  def test_bad(self):
    bad_samples = [
      "bob",
      "",
    ]
    self.verify_bad_samples(bad_samples)
    
if __name__ == "__main__":
  import unittest
  unittest.main()



#!/usr/bin/env python
# -*- encoding: latin1 -*-
#
# Copyright 2007 Scott Kirkwood

from _plugin import ClipboardPlugin, TestPlugin
import re

def create_plugin():
  return ForceToText()

class ForceToText(ClipboardPlugin):
  def __init__(self):
    ClipboardPlugin.__init__(self)
    self.plus_at_start = True
    
  def name(self):
    return 'Force clipboard text, only'
    
  def description(self):
    return 'Some applications support multiple clipboard format, force them to pick the text one'
    
  def convert(self, text):
    """ Convert Lines into Java strings
      
    Returns: Text
    """
    return self._ret_result("Forced to text", True, text)

class TestForceToText(TestPlugin):
  def setUp(self):
    self.instance = ForceToText()
    
  def test_good(self):
    good_samples = [
      ("String",
       'String'),
    ]
    self.verify_good_samples(good_samples, "Forced to text")
    
if __name__ == "__main__":
  import unittest
  unittest.main()



#!/usr/bin/env python
# -*- encoding: latin1 -*-
#
# Copyright 2007 Scott Kirkwood

from _plugin import ClipboardPlugin, TestPlugin
import re

def create_plugin():
  return Url2Python()
  
class Url2Python(ClipboardPlugin):
  def __init__(self):
    ClipboardPlugin.__init__(self)

  def name(self):
    return 'Convert an URL into python'
  
  def description(self):
    return 'Convert an URL into pretty printed python'

  def convert(self, text):
    """ Convert an http://summary.com:1234/graphnav?duration=4d&expr=too-high
    to 'http://summary.com:1234/graphnvav?' + urllib.urldecode(dict(
           duration = '4d',
           expr = 'too-high,
       ))
    Returns text (url)
    """
    import cgi
    import urllib
    
    query, params = urllib.splitquery(text)
    if not query:
      return self._ret_result("Couldn't parse URL", False, "")
    if not params:
      return self._ret_result("Created python", True, query)
    
    args = cgi.parse_qsl(params)
    result = ["'%s?' + urllib.urlencode(dict(" % query]
    for key, value in args:
      result.append("  %s = '%s'," % (key, value))
    result.append('))')
    return self._ret_result("Created python", True, '\n'.join(result))

class TestUrl2Python(TestPlugin):
  def setUp(self):
    self.instance = Url2Python()
    
  def test_good(self):
    good_samples = [
      ("http://summary.com:1234/graphnav?duration=4d&expr=too-high%3Dglobal%3Aprr_qps%3Alimit_rate10m.summary%3Blower-limit%3Dglobal%3Aprr_qps%3Amin_rate10m.summary%3Bupper-limit%3Dglobal%3Aprr_qps%3Amax_rate10m.summary%3Btotal%3Dglobal%3Aprr_qps%3Arate10m.summary%3Bformat%28zone%3Aprr_qps%3Arate10m.summary%2C%22%25zone%25%22%29&title=Bob+Storage+Server+Activity+%5BKqps%5D&using=%28%242*0.001%29&grid=&with_0=lines+lw+3&with_1=lines+lw+3&with_2=lines+lw+3&with_3=lines+lw+3&key=left&refresh=60",
          '\n'.join([
            "'http://summary.com:1234/graphnav?' + urllib.urlencode(dict(",
            "  duration = '4d',",
            "  expr = 'too-high=global:prr_qps:limit_rate10m.summary;lower-limit=global:prr_qps:min_rate10m.summary;upper-limit=global:prr_qps:max_rate10m.summary;total=global:prr_qps:rate10m.summary;format(zone:prr_qps:rate10m.summary,\"%zone%\")',",
            "  title = 'Bob Storage Server Activity [Kqps]',",
            "  using = '($2*0.001)',",
            "  with_0 = 'lines lw 3',",
            "  with_1 = 'lines lw 3',",
            "  with_2 = 'lines lw 3',",
            "  with_3 = 'lines lw 3',",
            "  key = 'left',",
            "  refresh = '60',",
            "))"])
      ),
      ('bob',
       'bob'
      ),
    ]
    self.verify_good_samples(good_samples, "Created python")

  def test_bad(self):
    bad_samples = [
      '',
    ]
    self.verify_bad_samples(bad_samples)
    
if __name__ == "__main__":
  import unittest
  unittest.main()



#!/usr/bin/env python

from ez_setup import use_setuptools
use_setuptools()
from setuptools import setup, find_packages

# Note to self:
# python setup.py sdist --formats=zip
# To create the zip file

# python setup.py --command-packages=setuptools.command bdist_egg
# To create the egg file

# python setup.py register
# to register with PyPI
# 

# create an egg and upload it
# setup.py register sdist bdist_egg upload

# Set this on command line
# DISTUTILS_DEBUG=true
# 
setup(
    name='clipboard-modifier',
    version='0.2.0',
    description="Change your clipboard text in a variety of ways",
    long_description=
"""A flexible system to modify the text in a clipboard in a variety of ways.
Out of the box we have:
  * Copy a spreadsheet and change the clipboard so that it can be pasted into a
  * wiki, with vertical bars (|) instead of tabs.  Modify your multi-line clipboard text so that it can be pasted into Java or Python as strings.
  * Modify an URL in the clipboard pointing to Amazon so that it has your Associate ID in it. 
  * Run a shell command piping the clipboard to it and retrieving the output from it.
  * Force a clibpboard to text (removing formatting, etc.).
  * Convert a complicated url into it's python equivalent, using urlencode.
  * And many more...

It uses wxPython and when you bring it's window to the front it modifies
the clipboard with the currently selected utility.
""",
    author='Scott Kirkwood',
    author_email='scottakirkwood@gmail.com',
    url='http://code.google.com/p/clipboard-modifier/',
    download_url='http://clipboard-modifier.googlecode.com/files/clipboard-modifier-0.2.0.zip',
    keywords=['clipboard', 'utility', 'wxPython', 'Python'],
    license='GNU GPL',
    platforms=['POSIX', 'Windows'],
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Topic :: Utilities',
    ], 
    packages=['clipboardmodifier', 'clipboardmodifier/plugins'],
    scripts=['scripts/clipboard-modifier'],
    zip_safe=False,
)



#!/usr/bin/env python
# -*- encoding: latin1 -*-
#
# Copyright 2007 Scott Kirkwood

import unittest
import wx

class ClipboardPlugin:
  def __init__(self):
    self.last_message = ''
    self.was_converted = False

    # These will be filled in by the parent
    self.parent = None
    self.filename = None
    
  def message(self):
    """ Short message describing what happened 
    
    Returns: String or empty string (not None)
    """
    return self.last_message
  
  def control_iter(self, dialog):
    yield None

  def get_state(self):
    """Return your state as a string."""
    return ''

  def restore_state(self, string):
    """Restore the state given a string"""
    pass

  def converted(self):
    """ Was the last convet() call successfull? 
    
    Returns: True/False
    """
    return self.was_converted
    
  def _ret_result(self, message, sucess, text=''):
    self.last_message = message
    self.was_converted = sucess
    return text

  def name(self):
    """ This is the name that appears in the dialog """
    pass
    
  def description(self):
    """ This is long description that shows up in tool tips """
    pass
    
  def convert(self, text):
    """ This is the function that does the actual convertion """
  
class TestPlugin(unittest.TestCase):
  def setUp(self):
    """ Setup self.instance here """
    pass
    
  def verify_good_samples(self, good_samples, expected_ok = None):
    """ Verify that an array of (intput, expected-output) match
    Ex
    """
    for good_sample, expected in good_samples:
      results = self.instance.convert(good_sample)
      self.assertTrue(self.instance.converted())
      if expected_ok != None:
        if not self.instance.message().startswith(expected_ok):
          self.fail("Expected: " + expected_ok + " but got " + self.instance.message())
      self.assertEquals(expected, results)

  def verify_bad_samples(self, bad_samples):
    for bad_sample in bad_samples:
      results = self.instance.convert(bad_sample)
      self.assertFalse(self.instance.converted())
      self.assertEquals('', results)
    



#!/usr/bin/env python
# -*- encoding: latin1 -*-
#
# Copyright 2007 Scott Kirkwood

from _plugin import ClipboardPlugin, TestPlugin
import re

def create_plugin():
  return DoNothing()

class DoNothing(ClipboardPlugin):
  def __init__(self):
    ClipboardPlugin.__init__(self)
    
  def name(self):
    return 'Leave clipboard alone'
    
  def description(self):
    return 'Leaves your clipboard intact'

  def message(self):
    return ''
  
  def converted(self):
    return False
  
  def convert(self, text):
    return ''


class TestDoNothing(TestPlugin):
  def setUp(self):
    self.instance = DoNothing()
    
  def test_good(self):
    samples = [
      "",
      "as;dfsdf",
    ]
    self.verify_bad_samples(samples)
    
if __name__ == "__main__":
  import unittest
  unittest.main()



#!/usr/bin/env python
# -*- encoding: latin1 -*-
#
# Copyright 2007 Scott Kirkwood

from _plugin import ClipboardPlugin, TestPlugin
import urllib
import wx

def create_plugin():
  return UrlToText()

class UrlToText(ClipboardPlugin):
  def __init__(self):
    ClipboardPlugin.__init__(self)
    self.unquote = True
    
  def name(self):
    return 'Translate %0A to text'
    
  def description(self):
    return 'Hard to understand %0A into text'
    
  def convert(self, text):
    """Convert URL text into something readable.
      
    Returns: Text
    """
    if self.unquote:
      new_text = urllib.unquote_plus(text)
    else:
      new_text = urllib.quote_plus(text)
    return self._ret_result("Unquoted", True, new_text)

  def control_iter(self, dialog):
    cb = wx.CheckBox(dialog, -1, label='Unquote')
    cb.SetValue(self.unquote)
    dialog.Bind(wx.EVT_CHECKBOX, self.EvtCheckBox, cb)
    yield cb

  def EvtCheckBox(self, event):
    self.unquote = event.Checked()
    self.parent.ModifyText()

class TestUrl2Text(TestPlugin):
  def setUp(self):
    self.instance = UrlToText()
    
  def test_good(self):
    good_samples = [
      ("%2f%7Econnolly%2f",
       '/~connolly/'),
      ("hello+there",
       "hello there"),
    ]
    self.verify_good_samples(good_samples, "Unquoted")
    
if __name__ == "__main__":
  import unittest
  unittest.main()
